#include <avr\interrupt.h>
#include <avr\io.h>
#include <util\delay.h>
#include "global.h"
#include "io_setup.h"
#include "config.h"
#include "rfm70.h"
#include "motors.h"
#include "timers.h"

Pwm* listOfPwms[PWM_MAX_NUMBER];
static uint8_t numberOfPwm = 0;

Motor* listOfMotors[MOTOR_MAX_NUMBER];
static uint8_t numberOfMotors = 0;

volatile uint8_t DirGlobal = 0;
volatile unsigned int loops = 0;
// Right
// [0] - horizontal ; [1] - vertical ;
// Left
// [2] - vertical ; [3] - horizontal ;
uint8_t inputData[PAYLOAD_LEN] = {0x80, 0x80, 0x80, 0x80};
uint8_t pipe;

void readConfig()
{
  #ifdef HAS_PWM1
  pwm1.pwmCompareUpdateCallback = pwm_A;
  pwm1.currentValue = &inputData[3];
  pwm1.prevValue = 0;
  listOfPwms[numberOfPwm] = &pwm1;
  ++numberOfPwm;
  #endif
  
  #ifdef HAS_MOTOR3
  motor3.currentValue = &inputData[1];
  motor3.dir = 0;
  motor3.limitValue = 0x80;
  motor3.left = M3_L;
  motor3.right = M3_R;
  motor3.stop = M3_L | M3_R;
  listOfMotors[numberOfMotors] = &motor3;
  ++numberOfMotors;
  #endif
}

int main()
{
  ioSetup();
  ioInt0Enable();
  readConfig();

  timer0_init();
  timer1_init();
  rfm70_rxSimpleTransmissionInit(PAYLOAD_LEN);
  sei();

  while(1)
  {
    for(int i = 0; i < numberOfPwm; ++i) {
      if (*listOfPwms[i]->currentValue != listOfPwms[i]->prevValue) {
        listOfPwms[i]->pwmCompareUpdateCallback(60 + (256 - *listOfPwms[i]->currentValue)/10);
        listOfPwms[i]->prevValue = *listOfPwms[i]->currentValue;
      }
    }

    for(int i = 0; i < numberOfMotors; ++i) {
      if (loops > (listOfMotors[i]->limitValue >> 1)) {
        DirGlobal &= ~listOfMotors[i]->stop;
      }
    }

    if(loops > 64) {
      loops = 0;

      for(int i = 0; i < numberOfMotors; ++i) {
        DirGlobal = (DirGlobal & ~(listOfMotors[i]->stop)) | listOfMotors[i]->dir;
      }
    }

    MOTOR_PORT = DirGlobal;
  }

  return 0;
}


SIGNAL(INT0_vect)
{
  cli();
  rfm70_receivePayload(inputData, &pipe);
  LED_PORT ^= 1 << LED0;
  sei();
}


SIGNAL(TIMER0_COMP_vect)
{
  cli();
  ++loops;
  countLimit(listOfMotors, numberOfMotors);
  LED_PORT ^= 1 << LED3;
  sei();
}
