#include <avr\io.h>
#include <avr\interrupt.h>
#include "timers.h"
#include "global.h"


/*====================================
		Initialization
====================================*/
void timer0_init(void)
{
	/* Timert 0 work in CTC mode */

	// Timer/Counter Configuration Register 0 A and B
	TCCR0 = (1 << WGM01)|(0 << WGM00)|(0 << COM01)|(0 << COM00)|(1 << CS02)|(0 << CS01)|(0 << CS00);
	
	// Timer/Counter value register
	TCNT0 = 0;

	// Timer/Counter Mask Register
	TIMSK |= (1 << OCIE0);

	// Timer/Counter Interrupt Flag Register
	TIFR &= ~(1 << OCF0);

	// OutputCompare value register
	OCR0 = 40;

}


void timer1_init(void)
{
	/* Timert 1 work in Fast PWM mode */

	// Timer/Counter Configuration Register A, B
	TCCR1A = (1 << COM1A1)|(0 << COM1A0)|(1 << COM1B1)|(0 << COM1B0)|(1 << WGM11)|(1 << WGM10);
	TCCR1B = (0 << WGM13)|(1 << WGM12)|(0 << CS12)|(1 << CS11)|(1 << CS10);
	
	// Timer/Counter value register
	TCNT1H = 0;
	TCNT1L = 0;

	// Timer/Counter Mask Register
	TIMSK &= ~((1 << OCIE1A)|(1 << OCIE1B));

	// Timer/Counter Interrupt Flag Register
	TIFR &= ~((1 << OCF1A)|(1 << OCF1A));

	// OutputCompare value registers
	OCR1AH = 0x00;
	OCR1AL = 0x5;

  OCR1BH = 0x00;
	OCR1BL = 0x5;	
  
  // 0x4 = 0.448ms
  // 0x27 = 2.56ms
}


void pwm_A(uint16_t val)
{
	// OutputCompare value register
	OCR1AH = val >> 8;
	OCR1AL = val;
}


void pwm_B(uint16_t val)
{
	// OutputCompare value register
	OCR1BH = val >> 8;
	OCR1BL = val;
}


void timers_on(void)
{
	// turn on interrupt enable
	TIMSK |= (1 << OCIE0);
}


void timer0_on(void)
{
	// turn on interrupt enable
	TIMSK |= 1 << OCIE0;
}


void timer0_off(void)
{
	// turn off interrupt enable
	TIMSK &= ~(1 << OCIE0);
}


void timer1_on(uint16_t prescaler)
{
	switch(prescaler)
	{
		case 0: prescaler = 0; break;
		case 1: prescaler = 1; break;
		case 8: prescaler = 2; break;
		case 32: prescaler = 3; break;
		case 64: prescaler = 4; break;
		case 128: prescaler = 5; break;
    case 256: prescaler = 6; break;
    case 1024: prescaler = 7; break;
		default: prescaler = 7;break;
	}
	
	// Set up prescaler
	TCCR1B = (TCCR1B & 0xf8)|(prescaler);
}


void timer1_off(void)
{
	// Set up prescaler
	TCCR1B = (TCCR1B & 0xf8)|(0 << CS12)|(0 << CS11)|(0 << CS10);
}
